@@ -16,7 +16,7 @@ class KSBoard : public Board
     uint nbRemovedLines(uint i) const { return linesRemoved[i]; }
 
  signals:
-    void removedLinesUpdated(uint nb, uint totalRemoved);
+    void removedLinesUpdated(uint i, uint nb);
 
  private:
     QMemArray<uint> filled;
